/****************************************************************
 ContentSensor.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "ContentSensor.H"
#include <iostream>
#include <fstream>
#include "MarkovChain.H"
#include "ThreePeriodicMarkovChain.H"
#include "FastMarkovChain.H"
#include "Fast3PMC.H"
#include "IMM.H"
#include "ThreePeriodicIMM.H"


TigrRegex ContentSensor::binmodRegex("binmod$");
const int ContentSensor::NO_PHASE=-1;


ContentSensor::ContentSensor()
{
}



void ContentSensor::setContentType(ContentType t)
{
  contentType=t;
}



void ContentSensor::setStrand(Strand s)
{
  strand=s;
}



ContentSensor *ContentSensor::loadBinary(const TigrString &filename)
{
  TigrFile file(filename);
  TigrString modelType;
  file >> modelType;
  file.rewind();

  if(modelType=="FastMC") return new FastMarkovChain(file);
  if(modelType=="Fast3PMC") return new Fast3PMC(file);

  throw TigrString("Unrecognized model type in ContentSensor::loadBinary(): ")+
    modelType;
}



ContentSensor *ContentSensor::load(const TigrString &filename)
{
  if(binmodRegex.search(filename))
    return loadBinary(filename);

  ifstream is(filename.c_str());
  if(!is.good())
    throw TigrString("Error opening file ")+filename+
      " in ContentSensor::load()";
  return load(is);    
}



ContentSensor *ContentSensor::load(istream &is)
{
  TigrString modelType;
  is >> modelType;

  if(modelType=="MC")
    return new MarkovChain(is);
  if(modelType=="3P")
    return new ThreePeriodicMarkovChain(is);
  if(modelType=="IMM")
    return new IMM(is);
  if(modelType=="3PIMM")
    return new ThreePeriodicIMM(is);

  /*
  if(modelType=="HMM")
    return new HiddenMarkovModel(is,alphabet);
  if(modelType=="NonstationaryMarkovChain")
    return new NonstationaryMarkovChain(is,alphabet);
  if(modelType=="CodingPotential")
    return new CodingPotential(is,alphabet);
  */

  throw TigrString("Unrecognized model type in ContentSensor::load(): ")+
    modelType;
}



ContentType ContentSensor::getContentType() const
{
  return contentType;
}



Strand ContentSensor::getStrand() const
{
  return strand;
}



bool ContentSensor::isCoding() const
{
  return ::isCoding(contentType);
}



bool ContentSensor::isIntron() const
{
  return ::isIntron(contentType);
}



bool ContentSensor::isIntergenic() const
{
  return ::isIntergenic(contentType);
}



bool ContentSensor::isUTR() const
{
  return ::isUTR(contentType);
}


