/****************************************************************
 ContentSensor.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_ContentSensor_H
#define INCL_ContentSensor_H

#include "tigr++/Sequence.H"
#include "tigr++/TigrString.H"
#include "tigr++/TigrRegex.H"
#include "ContentType.H"
#include "Strand.H"

class ContentSensor
{
  ContentType contentType; // like INITIAL-EXON, INTRON, 5'-UTR, etc.
  Strand strand;
  static TigrRegex binmodRegex;
  
  static ContentSensor *loadBinary(const TigrString &);
protected:
  ContentSensor();
  void setContentType(ContentType);
  void setStrand(Strand);
public:
  static const int NO_PHASE;

  // Static methods:
  static ContentSensor *load(const TigrString &filename);
  static ContentSensor *load(istream &is);

  // Virtual methods:
  virtual double scoreSingleBase(const Sequence &,const TigrString &,int index,
			  Symbol,char)=0;
  virtual void scoreSingleBase(const Sequence &,const TigrString &,int index,
			       Symbol,char,double &scorePhase0,
			       double &scorePhase1,
			       double &scorePhase2)=0;
  virtual double scoreSubsequence(const Sequence &,const TigrString &,int begin,
				  int length,int phase)=0;
  virtual ContentSensor *reverseComplement()=0;
  virtual bool save(const TigrString &filename)=0;
  virtual bool save(ostream &os)=0;
  virtual void useLogOdds(ContentSensor &nullModel)=0;
  virtual void useLogOdds_anonymous(ContentSensor &nullModel)=0;

  // Instance methods:
  virtual ~ContentSensor() {}
  ContentType getContentType() const;
  Strand getStrand() const;
  bool isCoding() const;
  bool isIntron() const;
  bool isIntergenic() const;
  bool isUTR() const;
};


#endif
