#ifndef INCL_ContentType_H
#define INCL_ContentType_H

#include <iostream>
#include "tigr++/TigrString.H"
#include "tigr++/TigrGffExon.H"
#include "Strand.H"
#include "SignalType.H"

enum ContentType
{
  UNKNOWN_CONTENT_FORWARD,
  UNKNOWN_CONTENT_REVERSE,
  INITIAL_EXON,
  INTERNAL_EXON,
  FINAL_EXON,
  SINGLE_EXON,
  INTRON,
  INTERGENIC,
  FIVE_PRIME_UTR,
  THREE_PRIME_UTR,
  NEG_INITIAL_EXON,
  NEG_INTERNAL_EXON,
  NEG_FINAL_EXON,
  NEG_SINGLE_EXON,
  NEG_INTRON,
  NEG_FIVE_PRIME_UTR,
  NEG_THREE_PRIME_UTR
};

ContentType reverseComplement(ContentType);
istream &operator>>(istream &,ContentType &);
ostream &operator<<(ostream &,const ContentType &);
Strand getStrand(ContentType);
TigrString contentTypeToString(ContentType);
TigrString contentTypeNiceString(ContentType);
ContentType stringToContentType(const TigrString &);
bool isCoding(ContentType);
bool isIntron(ContentType);
bool isIntergenic(ContentType);
bool isUTR(ContentType);
SignalType leftSignal(ContentType);
SignalType rightSignal(ContentType);
ContentType exonTypeToContentType(ExonType,Strand);

class ContentTypeInitializer
{
  static ContentTypeInitializer initializer;
  ContentTypeInitializer();
};

#endif
