/****************************************************************
 Fast3PMC.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Fast3PMC.H"
#include <iostream>
#include "MarkovChainCompiler.H"


Fast3PMC::Fast3PMC(ThreePeriodicMarkovChain &slowModel)
  : revComp(NULL)
{
  ContentType contentType=slowModel.getContentType();
  setContentType(contentType);
  setStrand(::getStrand(contentType));
  compileFrom(slowModel);
}



Fast3PMC::Fast3PMC(ContentType contentType)
  : revComp(NULL)
{
  setContentType(contentType);
  setStrand(::getStrand(contentType));
  chains[0]=chains[1]=chains[2]=NULL;
}



Fast3PMC::Fast3PMC(const TigrString &filename)
  : revComp(NULL)
{
  load(filename);
}



Fast3PMC::Fast3PMC(TigrFile &file)
  : revComp(NULL)
{
  load(file);
}



double Fast3PMC::scoreSingleBase(const Sequence &seq,const TigrString &str,
				 int index,Symbol s,char c)
{
  throw "attempt to use Fast3PMC on a non-periodic feature";
}



void Fast3PMC::scoreSingleBase(const Sequence &seq,const TigrString &str,int index,
			       Symbol s,char c,double &scorePhase0,
			       double &scorePhase1,double &scorePhase2)
{
  scorePhase0=chains[0]->scoreSingleBase(seq,str,index,s,c);
  scorePhase1=chains[1]->scoreSingleBase(seq,str,index,s,c);
  scorePhase2=chains[2]->scoreSingleBase(seq,str,index,s,c);
}



double Fast3PMC::scoreSubsequence(const Sequence &seq,const TigrString &str,
				  int begin,int length,int phase)
{
  char c;
  double score=0;
  int end=begin+length;
  for(int pos=begin ; pos<end ; ++pos)
    score+=scoreSingleBase(seq,str,pos,seq[pos],c);
  return score;
}



ContentSensor *Fast3PMC::reverseComplement()
{
  if(!revComp)
    {
      revComp=new Fast3PMC(::reverseComplement(getContentType()));
      revComp->revComp=this;
      for(int i=0 ; i<3 ; ++i)
	revComp->chains[i]=
	  static_cast<FastMarkovChain*>(chains[i]->reverseComplement());
    }
  return revComp;
}



bool Fast3PMC::save(const TigrString &filename)
{
  TigrFile file(filename,"w");
  return save(file);
}



bool Fast3PMC::save(ostream &os)
{
  throw "Attempt to save Fast3PMC into ASCII file -- please use binary files";
}



bool Fast3PMC::save(TigrFile &file)
{
  file << "Fast3PMC" << static_cast<int>(getContentType());
  for(int i=0 ; i<3 ; ++i)
    if(!chains[i]->save(file)) return false;
  return true;
}



void Fast3PMC::load(const TigrString &filename)
{
  TigrFile file(filename);
  load(file);
}



void Fast3PMC::load(TigrFile &file)
{
  TigrString modelType;
  file >> modelType;
  if(modelType!="Fast3PMC")
    throw TigrString("Attempt to load a model of type ")+modelType+
      " into Fast3PMC (optimized 3-period markov chain)";
  int contentType;
  file >> contentType;
  setContentType(static_cast<ContentType>(contentType));
  setStrand(::getStrand(static_cast<ContentType>(contentType)));
  for(int i=0 ; i<3 ; ++i)
    chains[i]=new FastMarkovChain(file);
}



void Fast3PMC::reset()
{
  for(int i=0 ; i<3 ; ++i)
    chains[i]->reset();
}



void Fast3PMC::compileFrom(ThreePeriodicMarkovChain &slowModel)
{
  for(int i=0 ; i<3 ; ++i)
    chains[i]=MarkovChainCompiler::compile(*slowModel.chains[i]);
}


