/****************************************************************
 Fast3PMC.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#ifndef INCL_Fast3PMC_H
#define INCL_Fast3PMC_H

#include "tigr++/Alphabet.H"
#include "tigr++/TigrFile.H"
#include "ContentSensor.H"
#include "FastMarkovChain.H"
#include "ThreePeriodicMarkovChain.H"

class Fast3PMC : public ContentSensor
{
  FastMarkovChain *chains[3];
  Fast3PMC *revComp;

  void load(const TigrString &filename);
  void load(TigrFile &);
  void compileFrom(ThreePeriodicMarkovChain &);
  Fast3PMC(ContentType);
public:
  Fast3PMC(ThreePeriodicMarkovChain &);
  Fast3PMC(const TigrString &filename);
  Fast3PMC(TigrFile &);
  virtual double scoreSingleBase(const Sequence &,const TigrString &,int index,
			  Symbol,char);
  virtual void scoreSingleBase(const Sequence &,const TigrString &,int index,
			       Symbol,char,double &scorePhase0,
			       double &scorePhase1,
			       double &scorePhase2);
  // NOTE: before calling scoreSubsequence() you should call reset() to
  // place the model into its initial state...unless you really
  // know what you're doing...
  virtual double scoreSubsequence(const Sequence &,const TigrString &,int begin,
				  int length,int phase);
  virtual ContentSensor *reverseComplement();
  virtual bool save(const TigrString &filename);
  virtual bool save(ostream &os);
  bool save(TigrFile &);
  virtual void useLogOdds(ContentSensor &nullModel) {throw "Fast3PMC::useLogOdds() not yet implemented";}
  virtual void useLogOdds_anonymous(ContentSensor &nullModel) {throw "Fast3PMC::useLogOdds_anonymous() not yet implemented";}
  void reset();
};


#endif
