/****************************************************************
 FastMarkovChain.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#ifndef INCL_FastMarkovChain_H
#define INCL_FastMarkovChain_H

#include "tigr++/TigrArray2D.H"
#include "tigr++/TigrArray1D.H"
#include "tigr++/Alphabet.H"
#include "tigr++/TigrFile.H"
#include "ContentSensor.H"
class MarkovChainCompiler;

class FastMarkovChain : public ContentSensor
{
  friend class MarkovChainCompiler;
  TigrArray2D<int> transitionMatrix;
  TigrArray1D<double> probabilityVector;
  TigrArray1D<int> statesPerOrder;
  int numStates;
  int order;
  int alphabetSize;
  int state;
  FastMarkovChain *revComp;

  void computeNumberOfStates();
  void load(const TigrString &filename);
  void load(TigrFile &);
  void loadNonRecurs(TigrFile &);
  int stringToState(const TigrString &s);
  TigrString stateToString(int state);
  bool saveNonRecurs(TigrFile &);
public:
  FastMarkovChain(int order,ContentType);
  FastMarkovChain(const TigrString &filename);
  FastMarkovChain(TigrFile &);
  virtual bool save(const TigrString &filename);
  virtual bool save(ostream &os); // don't use this
  bool save(TigrFile &);
  virtual double scoreSingleBase(const Sequence &,const TigrString &,int index,
			  Symbol,char);
  virtual void scoreSingleBase(const Sequence &,const TigrString &,int index,
			       Symbol,char,double &scorePhase0,
			       double &scorePhase1,
			       double &scorePhase2);

  // NOTE: before calling scoreSubsequence() you should call reset() to
  // place the model into its initial state...unless you really
  // know what you're doing...
  virtual double scoreSubsequence(const Sequence &,const TigrString &,int begin,
				  int length,int phase);

  virtual ContentSensor *reverseComplement();
  void setTrans(int state1,Symbol s,int state2);
  void setProb(int state,double prob);
  int statesOfOrder(int order);
  void reset();
  TigrString getState(); // useful for debugging
  void gotoState(const TigrString &);
  virtual void useLogOdds(ContentSensor &nullModel) {throw "FastMarkovChain::useLogOdds() not implemented yet";}
  virtual void useLogOdds_anonymous(ContentSensor &nullModel) {throw "FastMarkovChain::useLogOdds_anonymous() not implemented yet";}
};


#endif
