/****************************************************************
 GffReader.H

 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_GffReader_H
#define INCL_GffReader_H
#include "tigr++/TigrString.H"
#include "tigr++/TigrMap.H"
#include "tigr++/TigrVector.H"
#include "SignalSensor.H"
#include "GarbageCollector.H"
#include "ParseGraph.H"
#include "tigrscan.H"
class TigrScan;

class GffReader
{
public:
  typedef TigrVector<SignalPtr> SignalList;
  GffReader(const TigrString &filename,TigrVector<SignalSensor*> &,
	    GarbageCollector &);
  ~GffReader();
#ifdef EXPLICIT_GRAPHS
  ParseGraph *toGraph(TigrScan &);
#endif
  SignalList &getSignals();
  SignalList *signalsAt(int position);
  SignalPtr getSignal(int position,SignalType);
  SignalPtr getSignal(int position,SignalType,SignalPtr &pred);
  void addLeftTerminus(SignalPtr );
  void addRightTerminus(SignalPtr );
private:
  SignalList signals;
  TigrMap<int,SignalList*> posToSignals;
  TigrMap<SignalType,SignalSensor*> signalSensors;
  GarbageCollector &gc;

  void load(const TigrString &);
  void newSignal(int contextWindowPos,float score,SignalSensor &);
  void linkBack(SignalPtr);
  void createLeftTerminus(SignalType);
  int getPhase(SignalPtr pred);
};


#endif
