/**************************************************************
IMM.H
bmajoros@tigr.org 1/1/2003

TIGR++ : C++ class template library for bioinformatics

Copyright (c) 2003, The Institute for Genomic Research (TIGR),
Rockville, Maryland, U.S.A.  All rights reserved.
***************************************************************/
// ===========================================================
// IMM.H : Interpolated Markov Model
// bmajoros@tigr.org
// ===========================================================

#ifndef INCL_IMM_H
#define INCL_IMM_H

#include "tigr++/TigrVector.H"
#include "tigr++/TigrStringMap.H"
#include "tigr++/TigrChi2Table.H"
#include "tigr++/Sequence.H"
#include "tigr++/Alphabet.H"
#include "VariableLengthSubmodel.H"
#include <iostream.h>
#include "frame.H"
#include "strand.H"

class IMM : public VariableLengthSubmodel
{
  TigrVector<TigrStringMap<double>*> models; // indexed by order
  TigrVector<TigrStringMap<int>*> counts; // indexed by order
  int N, phase;
  Alphabet &alphabet;
  int alphabetSize, minSampleSize;
  TigrChi2Table chiSquaredTable;
  Strand strand;

  void buildModels(TigrVector<Sequence*> &);
  void updateCounts(TigrString &,int order);
  void undefine(TigrString &history,TigrStringMap<int> &model);
  double getScore(const char *,int position,int zero);
  void load(istream &);
  double interpolate(int order,const char *,int begin);
  IMM(Alphabet &,int N,int minSampleSize,int phase,char strand);
public:
  IMM(const IMM &);
  IMM(TigrVector<Sequence*> &,int N,Alphabet &,
      int minSampleSize,int phase=NO_PHASE);
  IMM(const TigrString &filename,Alphabet &);
  IMM(istream &,Alphabet &);
  ~IMM();
  double getLogP(Sequence &,const char *,int begin,int len);
  virtual bool save(ostream &);
  virtual bool save(const TigrString &);
  virtual void describe(ostream &);
  virtual void evaluate(TigrVector<ParseEdge*> &,Sequence &,
			const char *);
  virtual Submodel *reverseComplement();
  virtual Submodel *clone() const;
  virtual int getOrder() const {return N;}
};


#endif
