/****************************************************************
 IsochoreFile.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "IsochoreFile.H"
#include <iostream>
#include "tigr++/TigrFile.H"

// G+C <= 100% : tigrscan.cfg
TigrRegex IsochoreFile::lineRegex("G[+]C\\s*<=\\s*(\\S+)%\\s*:\\s*(\\S+)");


TigrConfigFile *IsochoreFile::parse(const TigrString &filename,
				    double gcContent,
				    TigrString &gffMessage)
{
  gcContent*=100;
  TigrFile infile(filename);
  while(!infile.eof())
    {
      TigrString line=infile.readLine();
      if(lineRegex.search(line))
	{
	  double upperBound=lineRegex[1].asDouble();
	  TigrString configFilename=lineRegex[2];
	  if(gcContent<=upperBound)
	    {
	      cerr << "\tG+C%=" << gcContent << " <= " << upperBound
		   << " -> using " << configFilename << endl;
	      gffMessage=TigrString("# G+C=")+int(10*gcContent)/10.0+
		"% -- used isochore file: "+configFilename;
	      return new TigrConfigFile(configFilename);
	    }
	}
    }
  throw TigrString("Error parsing ")+filename+" : G+C content = "+
    gcContent+"%";
}




