/****************************************************************
 MarkovChain.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#ifndef INCL_MarkovChain_H
#define INCL_MarkovChain_H

#include "ContentSensor.H"
#include "tigr++/TigrVector.H"
#include "tigr++/TigrStringMap.H"

class MarkovChain : public ContentSensor
{
  TigrVector<TigrStringMap<double>*> models; // indexed by order
  int N, phase;
  int alphabetSize, minSampleSize;

  void buildModels(TigrVector<Sequence*> &);
  void updateCounts(TigrString &,int order,int sequencePhase);
  void undefine(TigrString &history,TigrStringMap<double> &model);
  void load(istream &);
  MarkovChain(int order,int minSampleSize,int phase,Strand,ContentType);
public:
  MarkovChain(const TigrString &filename);
  MarkovChain(istream &);
  MarkovChain(TigrVector<Sequence*> &,int order,
	      int minSampleSize,int phase,ContentType);
  virtual ~MarkovChain();
  virtual double scoreSingleBase(Sequence &,const TigrString &,int index,
				 Symbol,char);
  virtual void scoreSingleBase(Sequence &,const TigrString &,int index,
			       Symbol,char,double &scorePhase0,
			       double &scorePhase1,
			       double &scorePhase2);
  virtual double scoreSubsequence(Sequence &,const TigrString &,int begin,
				  int length,int seqPhase);
  virtual ContentSensor *reverseComplement();
  virtual bool save(const TigrString &filename);
  virtual bool save(ostream &os);
  int getOrder() {return N;}
};


#endif
