/****************************************************************
 MarkovChain.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#ifndef INCL_MarkovChain_H
#define INCL_MarkovChain_H

#include "ContentSensor.H"
#include "tigr++/TigrVector.H"
#include "tigr++/TigrStringMap.H"
#include "TrainingSequence.H"

class MarkovChain : public ContentSensor
{
  TigrVector<TigrStringMap<double>*> models; // indexed by order
  int N, phase;
  int alphabetSize;
  MarkovChain *revComp;

  void revCompSeqs(TigrVector<TrainingSequence*>&,
		   TigrVector<TrainingSequence*>&);
  void buildModels(TigrVector<TrainingSequence*> &,int minSampleSize);
  void updateCounts_fw(TigrString &,int order,int sequencePhase,
		       int boostCount);
  void updateCounts_rev(TigrString &,int order,int seqPhase,
			int boostCount);
  void undefine_fw(TigrString &history,TigrStringMap<double> &model);
  void undefine_rev(TigrString &future,TigrStringMap<double> &model);
  void computeProbabilities_fw(int minSampleSize);
  void computeProbabilities_rev(int minSampleSize);
  void load(istream &);
public:
  MarkovChain(const MarkovChain &);
  MarkovChain(const TigrString &filename);
  MarkovChain(istream &,Strand=FORWARD_STRAND);
  MarkovChain(TigrVector<TrainingSequence*> &,int order,
	      int minSampleSize,int phase,ContentType,
	      Strand strand=EITHER_STRAND);
  virtual ~MarkovChain();
  virtual double scoreSingleBase(const Sequence &,const TigrString &,
				 int index,Symbol,char);
  virtual void scoreSingleBase(const Sequence &,const TigrString &,int index,
			       Symbol,char,double &scorePhase0,
			       double &scorePhase1,
			       double &scorePhase2);
  virtual double scoreSubsequence(const Sequence &,const TigrString &,int begin,
				  int length,int seqPhase);
  virtual ContentSensor *reverseComplement();
  virtual bool save(const TigrString &filename);
  virtual bool save(ostream &os);
  virtual void useLogOdds(ContentSensor &nullModel);
  virtual void useLogOdds_anonymous(ContentSensor &nullModel);
  int getOrder() {return N;}
};


#endif
