/**************************************************************
MddTree.H
bmajoros@tigr.org

Copyright (c) 2003, The Institute for Genomic Research (TIGR),
Rockville, Maryland, U.S.A.  All rights reserved.
***************************************************************/
#ifndef INCL_MddTree_H
#define INCL_MddTree_H
#include <iostream>
#include "tigr++/TigrString.H"
#include "tigr++/Alphabet.H"
#include "SignalSensor.H"
#include "TreeNode.H"

class MddTree : public SignalSensor
{
  TreeNode *root;
  SignalSensor *pooledModel;

  void load(istream &,GarbageCollector &);
public:
  MddTree(const MddTree &,GarbageCollector &,bool revComp);
  MddTree(TreeNode *root,SignalSensor *pooledModel,GarbageCollector &,
	  Strand strand,SignalType,double cutoff,int consensusLength,
	  int consensusOffset,int contextWindowLength);
  MddTree(istream &,GarbageCollector &);
  MddTree(const TigrString &filename,GarbageCollector &);
  virtual ~MddTree();
  void computeSubmodels(ModelBuilder &,ModelType,SignalType,
			int consensusOffset,int consensusLength,
			int contextWindowLength);
  int getMaxDepth();
  void printOn(ostream &) const;
  void describe(ostream &);

  virtual double getLogP(const Sequence &,const TigrString &,int begin);
  virtual SignalSensor *reverseComplement();
  virtual bool save(const TigrString &filename);
  virtual bool save(ostream &os);
  virtual SignalSensor *clone() const;

  virtual void useLogOdds(SignalSensor &nullModel) {}
  virtual void useLogOdds_anonymous(ContentSensor &nullModel) {}
};

ostream &operator<<(ostream &,MddTree &);


#endif
