
#include "ModelType.H"
#include "tigr++/TigrMap.H"
#include "tigr++/TigrString.H"

static TigrMap<TigrString,ModelType> modelTypeMap;
ModelTypeInitializer ModelTypeInitializer::initializer;


ModelTypeInitializer::ModelTypeInitializer()
{
  modelTypeMap["WMM"]=WMM_MODEL;
  modelTypeMap["WAM"]=WAM_MODEL;
  modelTypeMap["WWAM"]=WWAM_MODEL;
  modelTypeMap["3P"]=THREE_PERIODIC;
  modelTypeMap["MC"]=MARKOV_CHAIN;
  modelTypeMap["FastMC"]=MARKOV_CHAIN;
  modelTypeMap["Fast3PMC"]=MARKOV_CHAIN;
  modelTypeMap["IMM"]=IMM_MODEL;
  modelTypeMap["CB"]=CODON_BIAS;
  modelTypeMap["MDD"]=MDD;
  modelTypeMap["HMM"]=HMM_MODEL;
  modelTypeMap["NSMC"]=NONSTATIONARY_MC;
  modelTypeMap["3PIMM"]=IMM_3P;
}



TigrString toString(ModelType t)
{
  switch(t)
    {
    case WMM_MODEL: return "WMM";
    case WAM_MODEL: return "WAM";
    case WWAM_MODEL: return "WWAM"; 
    case THREE_PERIODIC: return "3P";
    case FAST_MC: return "FastMC";
    case FAST_3PMC: return "Fast3PMC";
    case MARKOV_CHAIN: return "MC";
    case IMM_MODEL: return "IMM";
    case CODON_BIAS: return "CB";
    case MDD: return "MDD";
    case SIGNAL_MODEL: return "SIGNAL_MODEL";
    case HMM_MODEL: return "HMM";
    case NONSTATIONARY_MC: return "NSMC";
    case IMM_3P: return "3PIMM";
    }
}



ModelType stringToModelType(const TigrString &s)
{
  if(!modelTypeMap.isDefined(s)) 
    throw TigrString("undefined ModelType: ")+s;
  return modelTypeMap[s];
}

