/**************************************************************
ScoreAnalyzer.H
bmajoros@tigr.org 1/1/2003

TIGR++ : C++ class template library for bioinformatics

Copyright (c) 2003, The Institute for Genomic Research (TIGR),
Rockville, Maryland, U.S.A.  All rights reserved.
***************************************************************/

// ScoreAnalyzer.H

#ifndef INCL_ScoreAnalyzer_H
#define INCL_ScoreAnalyzer_H

#include "tigr++/TigrVector.H"
#include <iostream>

enum ScoreType
  {
    POS_SCORE, // true positives & false negatives
    NEG_SCORE  // true negatives & false positives
  };

struct ScoreNode
{
  double score;
  ScoreType scoreType;
  ScoreNode(double score,ScoreType scoreType) : 
    score(score), scoreType(scoreType) {}
  bool operator<(ScoreNode other) { return score<other.score; }
  bool operator==(ScoreNode other) { return score==other.score; }
  bool operator>(ScoreNode other) { return score>other.score; }
};

struct PrecisionRecallPair
{
  double precision, recall, cutoff, percentFP, percentFN;
  PrecisionRecallPair(double p,double r,double cutoff,double percentFP,
		      double percentFN) : 
    precision(p), recall(r), cutoff(cutoff), percentFP(percentFP),
    percentFN(percentFN) {}
};

class ScoreAnalyzer
{
  TigrVector<PrecisionRecallPair> precRecallGraph;
  double bestAccuracy, bestCC;

  void analyze(TigrVector<double> &posScores,TigrVector<double> &negScores);
  void addToList(TigrVector<double> &,TigrVector<ScoreNode> &,ScoreType);
public:
  ScoreAnalyzer(TigrVector<double> &posScores,TigrVector<double> &negScores);
  void outputGraph(ostream &);
  double getMinRecallCutoff(double minRecall);
  double getPercentileCutoff(double percentile);//larger % -> higher cutoff
  void lookupCutoff(double cutoff,double &precision,double &recall);
  void lookupCutoff(double cutoff,double &precision,double &recall,
		    double &percentFP,double &percentFN);
  double getBestAccuracy() const;
  double getBestCC() const;
};


#endif
