/****************************************************************
 Signal.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Signal_H
#define INCL_Signal_H
#include <iostream>
#include "tigr++/TigrList.H"
#include "tigrscan.H"
#include "SignalType.H"
class Edge;
class SignalSensor;
class Propagator;
class GarbageCollector;

class Signal : public PointedTo
{
#ifdef EXPLICIT_GRAPHS
  TigrSet<Edge*> edgesIn;
  TigrSet<Edge*> edgesOut;
  int index; // index into graph's vertex array
#endif

  TigrSmartPointer<Signal> predecessors[3]; // optimal predecessors, one in each frame
  int contextWindowPosition;
  Propagator *propagators[2];
  SignalSensor &sensor;
  double signalScore; // score for the entire context window (not just cons.)
  double inductiveScores[3]; // includes signalScore!

  void initializePropagators(double signalScore);
public:
  Signal(int contextWindowPosition,double signalScore,SignalSensor &,
	 GarbageCollector &);
  //Signal(const Signal &s) : sensor(s.sensor) {throw "copy ctor";}
  //Signal &operator=(const Signal &) {throw "copy assign operator";}
  virtual ~Signal();
  Propagator &getPropagator(ContentType);
  void setPredecessor(int phase,TigrSmartPointer<Signal>);
  TigrSmartPointer<Signal> getPredecessor(int phase);
  int getContextWindowPosition();
  int getConsensusPosition() const;
  virtual int getContextWindowLength();
  virtual int getConsensusLength();
  int frameOfBaseFollowingConsensus();
  int posOfBaseFollowingConsensus();
  TigrSet<ContentType> &belongsInWhichQueues();
  TigrSet<ContentType> &linksBackToWhichQueues();
  Strand getStrand();
  SignalType getSignalType() const;
  double contextWindowScore(); // signal score, not the inductive score
  SignalSensor &getSensor();
  double &getInductiveScore(int phase);
  double priorInductiveScore(int phase); // not including signal score
  double posteriorInductiveScore(int phase); // including signal score
  double precedingFeatureScore(int phase); // =content+length+trans scores
  bool isLeftTerminus();
  void printOn(ostream &) const;

#ifdef EXPLICIT_GRAPHS
  bool isRightTerminus();
  void addEdgeIn(Edge *);
  void addEdgeOut(Edge *);
  TigrSet<Edge*> &getEdgesIn();
  TigrSet<Edge*> &getEdgesOut();
  void dropEdgeIn(Edge*);
  void dropEdgeOut(Edge*);
  inline int getIndex() {return index;}
  inline void setIndex(int x) {index=x;}
  Edge *findEdgeInFrom(TigrSmartPointer<Signal> pred);
#endif
};

ostream &operator<<(ostream &,const Signal &);

#endif
