/****************************************************************
 SignalQueue.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "SignalQueue.H"
#include <iostream>
#include "tigrscan.H"
#include "SignalTypeProperties.H"
#include "Accumulator.H"


IgnorantComparator SignalQueue::ignorantComparator;



void SignalQueue::addSignal(SignalPtr signal)
{
  ContentType contentType=getContentType();
  Propagator &propagator=signal->getPropagator(contentType);
  int propagatorPosition=propagator.getPosition();
  if(propagatorPosition <= accumulator.getPosition())
    bool inserted=mainQueue.push_back(signal);
  else
    holdingQueue.push_back(signal);
}



void SignalQueue::updateHoldingQueue(int position)
{
  // Checks whether any signals in holdingQueue have been passed by
  // the accumulator.  If so, calls flushAccumulator() and then moves
  // those signals into the main queue.

  ContentType contentType=getContentType();
  int accumulatorPosition=accumulator.getPosition();
  TigrList<SignalPtr >::iterator cur=holdingQueue.begin(), end=holdingQueue.end();
  bool first=true;
  for(; cur!=end ;)
    {
      SignalPtr signal=*cur;
      if(signal->getPropagator(contentType).getPosition() <= 
	 accumulatorPosition)
	{
	  if(first) {flushAccumulator();first=false;}
	  TigrList<SignalPtr >::iterator graduate=cur;
	  ++cur;
	  holdingQueue.erase(graduate);
	  SignalPtr displaced;
	  if(mainQueue.size()==mainQueue.getCapacity())
	    displaced=*mainQueue.begin();
	  bool inserted=mainQueue.insert(signal);
	  continue;
	}
      ++cur;
    }
}



TigrIterator<SignalPtr> &SignalQueue::begin()
{
  mainIter=mainQueue.begin();
  return mainIter;
}



TigrIterator<SignalPtr> &SignalQueue::end()
{
  mainEnd=mainQueue.end();
  return mainEnd;
}



int SignalQueue::numElements()
{
  return holdingQueue.size() + mainQueue.size();
}



void SignalQueue::drop(TigrIterator<SignalPtr > &victim)
{
  mainQueue.erase(static_cast<FSPLIterator<SignalPtr >&>(victim));
  SignalPtr signal=*victim;
}


