/****************************************************************
 SignalSensor.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#ifndef INCL_SignalSensor_H
#define INCL_SignalSensor_H

#include "tigr++/TigrStringMap.H"
#include "tigr++/TigrString.H"
#include "tigr++/Histogram.H"
#include "SignalType.H"
#include "Signal.H"
#include "Strand.H"
class ContentSensor;

class SignalSensor
{
  SignalType signalType; // like ATG, TAG, GT, AG, etc...
  int consensusLength, consensusOffset, contextWindowLength;
  Strand strand;
  double cutoff;
  TigrStringMap<char> consensuses;
  GarbageCollector &gc;
  Histogram<double> *probabilityInverter;//logP(seq|sig)->logP(sig|score)
protected:
  SignalSensor(GarbageCollector &);
  SignalSensor(GarbageCollector &,const SignalSensor &,
	       bool reverseComplement=false);
  SignalSensor(const SignalSensor &);
  void setSignalType(SignalType);
  void setSizes(int consensusLength,int consensusOffset,
		int contextWindowLength);
  void setStrand(Strand);
  GarbageCollector &getGC() {return gc;}
public:
  // Static methods:
  static SignalSensor *load(const TigrString &filename,GarbageCollector &);
  static SignalSensor *load(istream &is,GarbageCollector &);

  // Virtual methods:
  virtual SignalPtr detect(const Sequence &,const TigrString &,
			 int contextWindowPosition);
  virtual SignalPtr detectWithNoCutoff(const Sequence &,const TigrString &,
				       int contextWindowPosition);
  virtual SignalSensor *reverseComplement()=0;
  virtual double getLogP(const Sequence &,const TigrString &,int begin)=0;
  virtual bool save(const TigrString &filename)=0;
  virtual bool save(ostream &os)=0;
  virtual void addConsensus(const TigrString &);
  virtual SignalSensor *clone() const {throw "SignalSensor::clone()";}
  virtual void useLogOdds(SignalSensor &nullModel)=0;
  virtual void useLogOdds_anonymous(ContentSensor &nullModel)=0;

  // Instance methods:
  virtual ~SignalSensor() {}
  void useInverseProbabilities(Histogram<double> *);
  void multiplyCutoffBy(double);
  void ignoreCutoff();
  SignalType getSignalType() const;
  bool consensusOccursAt(const TigrString &,int index);
  void setCutoff(double);
  double getCutoff() const;
  Strand getStrand() const;
  TigrSet<ContentType> &belongsInWhichQueues();
  TigrSet<ContentType> &linksBackToWhichQueues();
  SignalPtr getLeftTerminus(double initialTransScore);
  SignalPtr getRightTerminus(int sequenceLength,double finalTransScore);
  int getContextWindowLength() const;
  int getConsensusOffset() const;
  int getConsensusLength() const;
  TigrStringMap<char> &getConsensuses() {return consensuses;}
};


#endif
