/****************************************************************
 SignalTypeProperties.H
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#ifndef INCL_SignalTypeProperties_H
#define INCL_SignalTypeProperties_H

#include "tigr++/TigrSet.H"
#include "tigr++/TigrMap.H"
#include "tigrscan.H"
#include "Strand.H"
#include "SignalType.H"
#include "ContentType.H"

class SignalTypeProperties
{
  struct SignalTypeInfo
  {
    TigrSet<int> allowablePhases;
    TigrSet<ContentType> belongsTo; // queues it can belong to
    TigrSet<ContentType> linksBackTo; // queues it can link back to
    bool codingConsensus;
    TigrMap<ContentType,int> propagatorIndexMap;
    Strand strand;
    SignalTypeInfo() : codingConsensus(false), strand(FORWARD_STRAND) {}
  };
  TigrMap<SignalType,SignalTypeInfo> info;
public:
  static SignalTypeProperties global;
  SignalTypeProperties();
  bool hasBeenLoaded() {return !info.isEmpty();}

  // Storing information about SignalTypes:
  SignalTypeProperties &allowPhase(SignalType,int phase);
  SignalTypeProperties &belongTo(SignalType,ContentType queueType);
  SignalTypeProperties &linkTo(SignalType,ContentType queueType);
  void setStrand(SignalType,Strand);
  void setConsensusCoding(SignalType,bool);

  // Querying information about SignalTypes:
  inline TigrSet<int> &getAllowablePhases(SignalType);
  inline TigrSet<ContentType> &belongsInWhichQueues(SignalType);
  inline TigrSet<ContentType> &linksBackToWhichQueues(SignalType);
  inline int whichPropagator(SignalType,ContentType);
  inline bool consensusBasesAreCoding(SignalType);
  inline Strand getStrand(SignalType);
  inline ContentType getContentType(SignalType from,SignalType to);
};



inline TigrSet<int> &SignalTypeProperties::getAllowablePhases(SignalType t)
{
  return info[t].allowablePhases;
}



inline TigrSet<ContentType> &SignalTypeProperties::belongsInWhichQueues(
   SignalType t)
{
  return info[t].belongsTo;
}



inline TigrSet<ContentType> &SignalTypeProperties::linksBackToWhichQueues(
   SignalType t)
{
  return info[t].linksBackTo;
}



inline int SignalTypeProperties::whichPropagator(SignalType t,ContentType c)
{
  return info[t].propagatorIndexMap[c];
}



inline bool SignalTypeProperties::consensusBasesAreCoding(SignalType t)
{
  return info[t].codingConsensus;
}



inline Strand SignalTypeProperties::getStrand(SignalType t)
{
  return info[t].strand;
}



inline ContentType SignalTypeProperties::getContentType(SignalType from,
							SignalType to)
{
  TigrSet<ContentType> lType=belongsInWhichQueues(from);
  TigrSet<ContentType> rType=linksBackToWhichQueues(to);
  TigrSet<ContentType> cType;
  lType.intersectWith(rType,cType);
  return *cType.begin();
}



#endif
