/****************************************************************
 TataCapModel.H

 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_TataCapModel_H
#define INCL_TataCapModel_H
#include "SignalSensor.H"
#include "ContentSensor.H"
#include "GarbageCollector.H"

class TataCapModel : public SignalSensor
{
  SignalSensor *tataModel, *capModel;
  ContentSensor *contentSensor; // ### Burge uses a 0th-order intergenic MC
  int minSeparation, maxSeparation; // between TATA & CAP context windows
  double durationScore;
  int tataLength, capLength;
  int minConsensusSeparation, maxConsensusSeparation;

  TataCapModel();
  SignalPtr detect_fw(const Sequence &,const TigrString &,int);
  SignalPtr detect_rev(const Sequence &,const TigrString &,int);
  void load(istream &);
  void load(const TigrString &filename);
public:
  TataCapModel(GarbageCollector &gc,SignalSensor *tataModel,
	       SignalSensor *capModel,
	       int minConsensusSeparation,int maxConsensusSeparation,
	       ContentSensor *inBetweenTataAndCap,SignalType signalType);
  TataCapModel(GarbageCollector &gc,const TigrString &filename);
  TataCapModel(GarbageCollector &gc,istream &);
  virtual SignalPtr detect(const Sequence &,const TigrString &,
			 int contextWindowPosition);
  virtual SignalSensor *reverseComplement();
  virtual double getLogP(const Sequence &,const TigrString &,int begin);
  virtual bool save(const TigrString &filename);
  virtual bool save(ostream &os);
  virtual void addConsensus(const TigrString &) {}
  virtual void useLogOdds(SignalSensor &nullModel) {throw "TataCapModel::useLogOdds() not yet implemented";}
  virtual void useLogOdds_anonymous(ContentSensor &nullModel) {throw "TataCapModel::useLogOdds_anonymous() not yet implemented";}
};


#endif
