/****************************************************************
 ThreePeriodicIMM.H
 3-Periodic Interpolated Markov Chains

 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#ifndef INCL_ThreePeriodicIMM_H
#define INCL_ThreePeriodicIMM_H

#include "ContentSensor.H"
#include "IMM.H"
class Fast3PMC;

class ThreePeriodicIMM : public ContentSensor
{
  friend class Fast3PMC;
  IMM *chains[3];
  
  void buildModels(TigrVector<TrainingSequence*> &,int minSampleSize,
		   int order);
  void load(istream &);
public:
  ThreePeriodicIMM(const TigrString &filename);
  ThreePeriodicIMM(istream &);
  ThreePeriodicIMM(Strand,ContentType);
  ThreePeriodicIMM(TigrVector<TrainingSequence*> &,int order,
			   int minSampleSize,ContentType);
  virtual ~ThreePeriodicIMM();
  virtual double scoreSingleBase(const Sequence &,const TigrString &,int index,
				 Symbol,char);
  virtual void scoreSingleBase(const Sequence &,const TigrString &,int index,
			       Symbol,char,double &scorePhase0,
			       double &scorePhase1,
			       double &scorePhase2);
  virtual double scoreSubsequence(const Sequence &,const TigrString &,int begin,
				  int length,int seqPhase);
  virtual ContentSensor *reverseComplement();
  virtual bool save(const TigrString &filename);
  virtual void useLogOdds(ContentSensor &nullModel);
  virtual void useLogOdds_anonymous(ContentSensor &nullModel);
  virtual bool save(ostream &os);
};


#endif
