/****************************************************************
 Transitions.C

 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Transitions.H"
#include <math.h>
#include <iostream>
#include <fstream>
#include "tigr++/Constants.H"
#include "tigrscan.H"


Transitions::Transitions(int numSignalTypes,const TigrString &filename,
			 float optimism,float intronOptimism)
  : matrix(numSignalTypes,numSignalTypes), optimism(optimism),
    intronOptimism(intronOptimism)
{
  matrix.setAllTo(NEGATIVE_INFINITY);
  ifstream is(filename.c_str());
  if(!is.good()) 
    throw filename+" : error opening file in Transitions::Transitions()";
  load(is);
}



Transitions::Transitions(int numSignalTypes,istream &is,float optimism,
			 float intronOptimism)
  : matrix(numSignalTypes,numSignalTypes), optimism(optimism),
    intronOptimism(intronOptimism)
{
  matrix.setAllTo(NEGATIVE_INFINITY);
  load(is);
}



void Transitions::load(istream &is)
{
  while(!is.eof())
    {
      TigrString line;
      getline(is,line);
      TigrVector<TigrString> &fields=*line.getFields(" \t");
      if(fields.size()==5)
	{
	  SignalType from=stringToSignalType(fields[0]);
	  SignalType to=stringToSignalType(fields[2]);
	  double P=fields[4].asDouble();
	  matrix[from][to]=log(P);
	}
      else if(fields.size()>0)
	throw TigrString("syntax error in transition file: ")+line;
      delete &fields;
    }
}



double Transitions::getLogP(SignalType from,SignalType to)
{
  if(beginsCoding(to)) 
    return 
      optimism +
      matrix[static_cast<int>(from)][static_cast<int>(to)];
  else if(beginsIntron(to))
    return 
      intronOptimism +
      matrix[static_cast<int>(from)][static_cast<int>(to)];    
  else
    return 
      matrix[static_cast<int>(from)][static_cast<int>(to)];
}

