/**************************************************************
TreeNode.H
bmajoros@tigr.org

Copyright (c) 2003, The Institute for Genomic Research (TIGR),
Rockville, Maryland, U.S.A.  All rights reserved.
***************************************************************/
#ifndef INCL_TreeNode_H
#define INCL_TreeNode_H
#include <iostream>
#include "tigr++/TigrVector.H"
#include "tigr++/Sequence.H"
#include "tigr++/Alphabet.H"
#include "SignalSensor.H"
#include "Partition.H"
#include "ModelBuilder.H"

class TreeNode
{
  TigrVector<TrainingSequence*> sequences;
  TreeNode *left, *right;
  Partition *partition;
  SignalSensor *submodel;

  void load(istream &,GarbageCollector &);
public:
  TreeNode(const TreeNode &); // does not copy over sequences
  TreeNode(Partition *);
  TreeNode(istream &,GarbageCollector &);
  TreeNode(const TigrString &filename,GarbageCollector &);
  virtual ~TreeNode();
  bool hasPartition();
  TigrVector<TrainingSequence*> &getSequences();
  TreeNode *getLeftChild();
  TreeNode *getRightChild();
  void setLeftChild(TreeNode *);
  void setRightChild(TreeNode *);
  Partition &getPartition();
  void addSequence(TrainingSequence *);
  void addSequences(TigrVector<TrainingSequence*> &);
  void printOn(ostream &) const;
  void printOn(ostream &,int level) const;
  bool save(const TigrString &filename);
  bool save(ostream &);
  void computeSubmodels(ModelBuilder &,ModelType,SignalType,
			int consensusOffset,int consensusLength);
  double getLogP(const Sequence &,const char *,int begin);
  SignalSensor *getSubmodel();
  void getSubmodels(TigrVector<SignalSensor*> &);
  int getMaxDepth();
  TreeNode *reverseComplement(int contextWindowLength);
  TreeNode *clone() const;
};

ostream &operator<<(ostream &,const TreeNode &);

#endif
