/**************************************************************************
 WWAM.H : Windowed Weight Array Matrix (WAM w/probabilities averaged over
                                        a small window)

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_WWAM_H
#define INCL_WWAM_H
#include "tigr++/TigrString.H"
#include "tigr++/TigrVector.H"
#include "MarkovChain.H"
#include "SignalSensor.H"
#include "TrainingSequence.H"
#include "GarbageCollector.H"

class WWAM : public SignalSensor
{
  TigrVector<MarkovChain *> matrix;

  void load(istream &);
public:
  WWAM(GarbageCollector &,const WWAM &, bool reverseComplement=false);
  WWAM(GarbageCollector &,TigrString &filename);
  WWAM(GarbageCollector &,istream &);
  WWAM(GarbageCollector &,TigrVector<TrainingSequence*> &, int order, 
       int minSampleSize,SignalType,int consensusOffset,int consensusLength); 
       // training constructor
  virtual ~WWAM();
  virtual bool save(const TigrString &filename);
  virtual bool save(ostream &os);
  double getLogP(const Sequence &,const TigrString &,int begin);
  virtual SignalSensor *reverseComplement();
  virtual void useLogOdds(SignalSensor &nullModel);
  virtual void useLogOdds_anonymous(ContentSensor &nullModel);
};

#endif
