
// build-tata-cap-model.C

#include <string>
#include <iostream>
#include "tigr++/TigrCommandLine.H"
#include "tigr++/DnaAlphabet.H"
#include "tigrscan.H"
#include "TataCapModel.H"

int frame;
Alphabet alphabet;

class Application
{
public:
  Application();
  int main(int argc,char *argv[]);
};


int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
  {
    // Process command line
    TigrCommandLine cmd(argc,argv,"");
    if(cmd.numArgs()!=9)
      throw TigrString(
"\nbuild-tata-cap-model <tata.model> <cap.model> <intervening-content.model>\n\
   <min-consensus-separation> <max-consensus-separation> <signal-type>\n\
   <output-filestem> <TATAAA,TATATA,CATAAA,CATATA> <CA>\n\
\n\
   <signal-type> = ATG|TAG|GT|AG|PROMOTER|POLYA\n");
    TigrString tataFilename=cmd.arg(0);
    TigrString capFilename=cmd.arg(1);
    TigrString contentModelFilename=cmd.arg(2);
    int minConsensusSeparation=cmd.arg(3).asInt();
    int maxConsensusSeparation=cmd.arg(4).asInt();
    SignalType signalType=::stringToSignalType(cmd.arg(5));
    TigrString filestem=cmd.arg(6);
    TigrString tataConsensuses=cmd.arg(7);
    TigrString capConsensuses=cmd.arg(8);
    alphabet=DnaAlphabet::global;

    // Load submodels
    SignalSensor *tataModel=SignalSensor::load(tataFilename);
    SignalSensor *capModel=SignalSensor::load(capFilename);
    ContentSensor *contentModel=ContentSensor::load(contentModelFilename);

    // Install consensus strings
    TigrVector<TigrString> *tataCons=tataConsensuses.getFields("|,:./");
    for(int i=0 ; i<tataCons->size() ; ++i)
      tataModel->addConsensus((*tataCons)[i]);
    TigrVector<TigrString> *capCons=capConsensuses.getFields("|,:./");
    for(int i=0 ; i<capCons->size() ; ++i)
      capModel->addConsensus((*capCons)[i]);

    // Create composite Tata-Cap model & save to file
    TataCapModel *model=
      new TataCapModel(tataModel,capModel,minConsensusSeparation,
		       maxConsensusSeparation,contentModel,signalType);
    TigrString outfile=filestem+".model";
    model->save(outfile);

    return 0;
  }

