
// compile-markov-chain.C

#include <string>
#include <iostream>
#include "tigr++/TigrCommandLine.H"
#include "tigr++/DnaAlphabet.H"
#include "tigr++/TigrRegex.H"
#include "MarkovChain.H"
#include "FastMarkovChain.H"
#include "MarkovChainCompiler.H"

Alphabet alphabet;

class Application
{
  TigrRegex filenameRegex;
public:
  Application();
  int main(int argc,char *argv[]);
};


int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  : filenameRegex("([^/]+).model$")
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
  {
    // Process command line
    TigrCommandLine cmd(argc,argv,"");
    if(cmd.numArgs()!=1)
      throw string("compile-markov-chain <*.model>       {writes to *.binmod}");
    TigrString infile=cmd.arg(0);
    if(!filenameRegex.search(infile))
      throw TigrString("Can't parse filename: ")+infile;
    TigrString outfile=filenameRegex[1]+".binmod";

    // Misc. initialization
    alphabet=DnaAlphabet::global;
    MarkovChain *mc=new MarkovChain(infile);

    // Perform the compilation
    FastMarkovChain *fmc=MarkovChainCompiler::compile(*mc);

    // Save the resulting compiled model
    TigrFile file(outfile,"w");
    fmc->save(file);

    return 0;
  }

