/****************************************************************
 entropy.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include "tigr++/TigrString.H"
#include "tigr++/TigrCommandLine.H"
#include "tigr++/TigrSequenceEntropy.H"
#include "tigr++/TigrFastaReader.H"
#include "tigr++/TigrSummaryStats.H"
using namespace std;


class Application
{
public:
  Application();
  int main(int argc,char *argv[]);
};


int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
  {
    // Process command line
    TigrCommandLine cmd(argc,argv,"");
    if(cmd.numArgs()!=1)
      throw string("entropy <*.fasta>");
    TigrString filename=cmd.arg(0);

    // Process all sequences in FASTA file
    TigrVector< TigrVector<double> > samples(9);
    TigrFastaReader reader(filename);
    TigrString def, seq;
    double maxEntropy;
    while(reader.nextSequence(def,seq))
      {
	if(seq.length()<40) continue;
	for(int order=0 ; order<=8 ; ++order)
	  {
	    double H=TigrSequenceEntropy::entropy(seq,order,maxEntropy);
	    if(H==0) continue;
	    H/=maxEntropy;
	    samples[order].push_back(H);
	  }
      }

    // Compute means
    TigrVector<double> curves[3];
    for(int order=0 ; order<=8 ; ++order)
      {
	TigrSummaryStats stats(samples[order]);
	double sigma=stats.getStdDev();
	double mu=stats.getMean();
	curves[0].push_back(mu-sigma);
	curves[1].push_back(mu);
	curves[2].push_back(mu+sigma);
      }

    // Produce output
    for(int i=0 ; i<3 ; ++i)
      {
	TigrVector<double> &v=curves[i];
	for(int order=0 ; order<=8 ; ++order)
	  cout<<order<<"\t"<<v[order]<<endl;
	cout<<endl;
      }

    return 0;
  }

