/****************************************************************
 tigrscan
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "TigrScan.H"
#include "tigrscan.H"
#include "EdgeFactory.H"
#include "tigr++/TigrFastaReader.H"
#include "tigr++/Constants.H"
using namespace std;

#ifdef EXPLICIT_GRAPHS
//#error Please edit the file tigrscan.H, comment out the definition of EXPLICIT_GRAPHS, issue a "make clean", and recompile this project
#endif

#ifdef FORCE_SPECIFIC_SIGNALS
#error Please edit the file tigrscan.H, comment out the definition of FORCE_SPECIFIC_SIGNALS, issue a "make clean", and recompile this project
#endif

static const char *PROGRAM_NAME="tigrscan";
static const char *VERSION="1.0";
Alphabet alphabet;
int frame; // ### CAUTION: this is required by older code; to be removed

void AppMain(int argc,char *argv[]);

int main(int argc,char *argv[])
  {
    try
      {
	AppMain(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



void AppMain(int argc,char *argv[])
{
  // Process command line
  TigrCommandLine cmd(argc,argv,"s:");
  if(cmd.numArgs()!=2)
    throw TigrString(
    "\ntigrscan <*.iso> <*.fasta> [options]\n\
       options:\n\
          -s <N> : ignore sequences in FASTA shorter than N bases\n");
  TigrString isochoreFilename=cmd.arg(0);
  TigrString fastaFilename=cmd.arg(1);
  alphabet=DnaAlphabet::global;
  int minSeqLen=
    (cmd.option('s') ? cmd.optParm('s').asInt() : 1);

  // Process all the contigs
  TigrFastaReader fastaReader(fastaFilename);
  TigrString defline, seqString;
  float gcContent;
  int transcriptId=999;
  while(fastaReader.nextSequence(defline,seqString))
    {
      Sequence seq(seqString,DnaAlphabet::global);
      if(seq.getLength()<minSeqLen) continue;

      // Parse the substrate ID out of the defline
      TigrString contigId, remainder;
      TigrFastaReader::parseDefline(defline,contigId,remainder);
      cerr<<"processing substrate "<<contigId<<"..."<<endl;

      // Predict genes and get path
      cerr<<"identifying open reading frames..."<<endl;
      EdgeFactory edgeFactory;
      TigrScan tigrscan(PROGRAM_NAME,VERSION,edgeFactory,transcriptId);
      TigrStack<SignalPtr> *path=
	tigrscan.mainAlgorithm(seq,seqString,isochoreFilename,contigId);

      // Don't need the path; just delete it
      delete path;
    }
}
